// sysinfo.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "stdio.h"

static unsigned short alphaMMXmul_const1[4] = {256,256,256,256};
static unsigned alphaMMXmul_0[2] = {1,1};

void MixAlphaMMX32(void *dest,const void *src,unsigned len,unsigned opacity) {

__asm {
	mov		edi,dest
	mov		ebx,src
	mov		ecx,len
	mov		edx,opacity

    push    ebp

    movzx   eax,dl
    movq    mm7,[alphaMMXmul_const1]

    shl     eax,16
    shr     ecx,1
    add     eax,edx
    mov     [alphaMMXmul_0],eax
    mov     [alphaMMXmul_0 + 4],eax
    movq    mm6,[alphaMMXmul_0]
    pxor    mm5,mm5
    psubusw mm7,mm6

;mm0 00000000 00000000 00000000 rrrrrrrr 00000000 gggggggg 00000000 bbbbbbbb

ALIGN 16
MixAlphaMMX32_MainLoop:

    movd    mm0,[edi]
    add     edi,4

    movd    mm1,[ebx]
    add     ebx,4

    movd    mm2,[edi]
    punpcklbw mm0,mm5

    movd    mm3,[ebx]
    punpcklbw mm1,mm5

    pmullw  mm0,mm6
    punpcklbw mm2,mm5

    pmullw  mm1,mm7
    punpcklbw mm3,mm5

    paddusw mm0,mm1
    pmullw  mm2,mm6

    psrlw   mm0,8
    pmullw  mm3,mm7

    packuswb mm0,mm0
    paddusw mm2,mm3

    movd    [edi-4],mm0
    psrlw   mm2,8

    add     ebx,4
    packuswb mm2,mm2

    movd    [edi],mm2
    add     edi,4

	dec		ecx
    jnz    MixAlphaMMX32_MainLoop

    emms
    pop ebp
}
}


static char CPUident[13];

#pragma warning( disable : 4035)
int whatCPU(void) {
__asm {
	mov  CPUident[12],0		// string describing CPU manufacturer
	mov  ebx,386			// starting with i386
	pushfd
	pop  eax
	mov  ecx,eax
	mov  edx,eax
	xor  eax,40000h			// 386 won't change EFALGS bit 34
	push eax
	popfd
	pushfd
	pop  eax
	xor  eax,ecx
	je   CPUexit
	add  ebx,100
	mov  eax,edx
	mov  ecx,eax
	xor  eax,200000h		// most 486 won't change EFALGS bit 37
	push eax				// (that is, won't support CPUID)
	popfd
	pushfd
	pop  eax
	xor  eax,ecx
	je   CPUexit
	mov  eax,0
/* 586 specific code */
	cpuid					// obtain CPU stepping from CPUID
	mov DWORD PTR [CPUident+0],ebx
	mov DWORD PTR [CPUident+4],edx
	mov DWORD PTR [CPUident+8],ecx
	mov  eax,1
	cpuid
	xor  ebx,ebx
	mov  bl,ah
	mov  eax,100
	mul  ebx
	add  eax,86
	mov  ebx,eax

CPUexit:
	mov  eax,ebx
}
	/* return eax; */
}

int whatCPUExtensions(void) {
__asm {
	mov   edi,0
    pushfd
    pop   eax
    mov   edx,eax
    xor   eax,200000h
    push  eax
    popfd
    pushfd
    pop   eax
    and   eax,200000h
    and   edx,200000h
    cmp   eax,edx
    je    ExtensionsEnd
    mov   eax,1
/* 586 specific code */
    cpuid
    test  edx,00800000h		// Test bit 23, for MMX existence.
    je    ExtensionsNoMMX
    or    edi,1
ExtensionsNoMMX:
	test  edx, 02000000h	// Test bit 25, for Streaming SIMD Extensions existence.
    je    ExtensionsNoKNI
    or    edi,4
ExtensionsNoKNI:
	mov   eax,080000000h	// Test CPUID bit 32, for 3DNow existence.
    cpuid
    test  eax,eax
    jz    ExtensionsEnd
    mov   eax,080000001h
    cpuid
    test  edx,080000000h
    je    ExtensionsEnd
    or    edi,2
ExtensionsEnd:
	mov	  eax,edi
}
	/* return eax; */
}

static WORD NPXtemp;

int whatNPX(void) {
__asm {
	fninit
	mov  ebx,OFFSET NPXtemp
	mov  WORD PTR [ebx],078a9h
	fnstsw [ebx]
	cmp  WORD PTR [ebx],0
	jne  WNPXnone
	fnstcw [ebx]
	mov  ax,[ebx]
	and  ax,103Fh
	cmp  ax,3Fh
	jne  WNPXnone
	fld1
	fldz
	fdiv
	fld  ST
	fchs
	fcompp
	fstsw [ebx]
	mov  ax,[ebx]
	sahf
	je   WNPX287
	mov  eax,387
	jmp  WNPXexit
WNPX287:
	mov  eax,287
	jmp  WNPXexit
WNPXnone:
	xor  eax,eax
WNPXexit:
}
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	char tmp[256];
	SYSTEM_INFO si;
	MEMORYSTATUS ms;

	int cpu = whatCPU();
	int npx = whatNPX();
	int cpuEx = whatCPUExtensions();

	GetSystemInfo(&si);
	GlobalMemoryStatus(&ms);

	sprintf(tmp,"CPU model: %d (%d)\nNPU: %d",cpu,si.dwProcessorType,npx);
	MessageBox(0,tmp,"CPU Information",MB_OK);

	sprintf(tmp,"CPU extensions: %s. %s, %s\n",
		(cpuEx & 1) ? "MMX" : "no MMX",
		(cpuEx & 2) ? "3DNow!" : "no 3DNow!",
		(cpuEx & 4) ? "KNI" : "no KNI");
	MessageBox(0,tmp,"CPU Information",MB_OK);

	const float meg = 1024.0*1024.0;
	sprintf(tmp,"Physical memory in MB (total/free): %.1f / %.1f\n"
		"Virtual memory in MB (total/free): %.1f / %.1f\n",
		ms.dwTotalPhys/meg,ms.dwAvailPhys/meg,ms.dwTotalVirtual/meg,ms.dwAvailVirtual/meg);
	MessageBox(0,tmp,"RAM Information",MB_OK);

	return 0;
}



